function u = chanEsodoglu(R)
    
    % some interesting values for R: 10, 50, 100, 150
    if nargin == 0
        R = 50;
    end
    
    nu = 2;
    dt = 0.01;
    iters = 500;        % need a lot of iterations > 1000 for large R >= 100
    tv_eps = 1e-1;      % small values for epsilon << 1e-1 produce noisy boundaries
    thresh  = 0.5;      % threshold for binarization for display
    
    close all;

    I       = double(imread('image.pgm'))/255;
    [m,n]   = size(I);
    
    % initialization with disk of radius R
    center  = floor([m/2 n/2]);
    [X, Y]  = meshgrid(1:m,1:n);
    u       = project(R - sqrt((X-center(1)).^2 + (Y-center(2)).^2));
    
    figure; colormap gray;
    
    for i=1:iters
        
        [mu1, mu2] = approxRegions(u,I);
        du = update(u,I,nu,mu1,mu2,tv_eps);
        u  = project(u-dt*du);
        
        if mod(i,10) == 0
            display(I, u, thresh)
        end
        
        fprintf('Iteration: %d\n',i);
    end
    
    display(I, u, thresh);

end


function display(I, u, thresh)
    J = I;
    J(u<thresh) = J(u<thresh) * 0.5;
    subplot(1,2,1);
    imagesc(J), axis image
    subplot(1,2,2);
    imagesc(u), axis image
    drawnow;
end



function [mu1,mu2] = approxRegions(u,I)
    mu1 = mean(I(u>0.5));
    mu2 = mean(I(u<0.5));
end


function du = update(u,I,nu,mu1,mu2,eps)
    ux    = dxp(u);
    uy    = dyp(u);

    scale = 1./sqrt(ux.^2+uy.^2+eps^2);

    ux    = ux.*scale;
    uy    = uy.*scale;
    uxx   = dxm(ux);
    uyy   = dym(uy);
    du    = (I-mu1).^2-(I-mu2).^2-nu*(uxx+uyy);
end


function u = project(f)
    u=f;
    u((u>1))=1;
    u((u<0))=0;
end