function u = chanEsodogluPrimalDual(R)
    
    % some interesting values for R: 10, 50, 100, 150
    if nargin == 0
        R = 50;
    end
    
    nu = 2;             % boundary length penalty
    sigma = 0.2;        % step-size of dual variable update
    tau = 0.2;          % step-size of primal variable update
    iters = 500;        % converges faster than gradient descent impl.
    thresh  = 0.5;      % threshold for binarization for display
    
    close all;

    I       = double(imread('image.pgm'))/255;
    [m,n]   = size(I);
    
    % initialization with disk of radius R
    center  = floor([m/2 n/2]);
    [X, Y]  = meshgrid(1:m,1:n);
    u       = project_u(R - sqrt((X-center(1)).^2 + (Y-center(2)).^2 + 0.01));
    u_bar   = u;
    xi      = zeros(m,n,2);
    
    figure; colormap gray;
    
    for i=1:iters
        
        [mu1, mu2] = approxRegions(u,I);
        u_prev = u;
        
        dxi = update_xi(u_bar);
        xi  = project_xi(xi + sigma*dxi);
        
        du  = update_u(xi, I, nu, mu1, mu2);
        u   = project_u(u - tau*du);
        
        u_bar = 2*u - u_prev;
        
        if mod(i,10) == 0
            display(I, u, thresh);
        end
        
        fprintf('Iteration: %d\n',i);
    end
    
    display(I, u, thresh);

end


function display(I, u, thresh)
    J = I;
    J(u<thresh) = J(u<thresh) * 0.5;
    subplot(1,2,1);
    imagesc(J), axis image
    subplot(1,2,2);
    imagesc(u), axis image
    drawnow;
end


function [mu1,mu2] = approxRegions(u,I)
    mu1 = mean(I(u>0.5));
    mu2 = mean(I(u<0.5));
end


function dxi = update_xi(u_bar)
    ux = dxp(u_bar);
    uy = dyp(u_bar);
    dxi = cat(3, ux, uy);
end


function du = update_u(xi, I, nu, mu1, mu2)
    xix    = dxm(xi(:,:,1));
    xiy    = dym(xi(:,:,2));
    du    = ((I-mu1).^2-(I-mu2).^2)/nu - (xix+xiy);
end


function u = project_u(f)
    u=f;
    u((u>1))=1;
    u((u<0))=0;
end


function xi = project_xi(v)
    scale = max(1, sqrt(v(:,:,1).^2 + v(:,:,2).^2));
    scale = cat(3, scale, scale);
    xi = v ./ scale;
end